﻿namespace wmaenc
{
    using System;
    using Microsoft.Expression.Encoder;
    using Microsoft.Expression.Encoder.Profiles;
    using CommandLine;
    using CommandLine.Text;
    using System.Text;
    using System.IO;
    using System.Linq;
    using System.Reflection;
    using System.Resources;

    public enum WmaCodec
    {
        wma = AudioCodec.Wma,
        wmap = AudioCodec.WmaProfessional
    }

    public enum Mode
    {
        cbr, vbr, abr
    }

    public class Options
    {
        [Option('i', "input", Required = true, HelpText = "WAV file to encode")]
        public string InputFile { get; set; }

        [Option('c', "codec", HelpText = "Codec (wma or wmap)")]
        public Nullable<WmaCodec> Codec { get; set; }

        [Option('o', "output", HelpText = "WMA filename to output")]
        public string OutputFile { get; set; }

        [Option('m', "mode", HelpText = "Mode (cbr, abr or vbr)")]
        public Nullable<Mode> Mode { get; set; }

        [Option('b', "bitrate", HelpText = "Bitrate (for cbr and abr mode)")]
        public Nullable<int> Bitrate { get; set; }

        [Option('q', "quality", HelpText = "VBR Quality (0-100)")]
        public Nullable<int> Quality { get; set; }

        [Option('r', "rate", HelpText = "Sample rate in KHz")]
        public Nullable<int> Rate { get; set; }

        [Option("bits", HelpText = "Bits per sample")]
        public Nullable<int> Bits { get; set; }

        [Option("channels", HelpText = "Number of channels")]
        public Nullable<int> Channels { get; set; }

        [HelpOption]
        public string GetUsage()
        {
            return wmaenc.Properties.Resources.help;
        }
    }


    public class Program
    {
        public Program(Options o)
        {
            int bitrate;
            int bits;
            int channels;
            int rate;
            int quality;

            if (o.Codec == null)
                o.Codec = WmaCodec.wma;

            if (o.Mode == null)
                o.Mode = Mode.cbr;

            if (o.Bitrate == null)
                bitrate = 128;
            else
                bitrate = (int)o.Bitrate;

            if (o.Channels == null)
                channels = 2;
            else
                channels = (int)o.Channels;

            if (o.Rate == null)
                rate = 44100;
            else
                rate = (int)o.Rate;

            if (o.Bits == null)
                bits = 16;
            else
                bits = (int)o.Bits;

            if (o.Quality == null)
                quality = 80;
            else
                quality = (int)o.Quality;

            var j = new Job();
            var m = new MediaItem(o.InputFile);
            var a = new WmaAudioProfile();
            a.Codec = (AudioCodec)o.Codec;

            Bitrate b = null;

            if (o.Bits == null && o.Codec == WmaCodec.wmap)
            {
                switch (o.Mode)
                {
                    case Mode.cbr:
                        if (o.Bitrate > 256)
                            bits = 24;
                        else
                            bits = 16;
                        break;
                    case Mode.abr:
                        if (o.Bitrate <= 64)
                            bits = 16;
                        else
                            bits = 24;
                        break;
                    case Mode.vbr:
                        bits = 24;
                        break;
                }
            }

            switch (o.Mode)
            {
                case Mode.abr:
                    b = new VariableUnconstrainedBitrate(bitrate);
                    break;
                case Mode.cbr:
                    b = new ConstantBitrate(bitrate);
                    break;
                case Mode.vbr:
                    b = new VariableQualityBitrate(quality);
                    break;
            }

            a.Bitrate = b;
            a.BitsPerSample = bits;
            a.Channels = channels;
            a.SamplesPerSecond = rate;

            var f = new WindowsMediaOutputFormat();

            if (o.OutputFile == null)
            {
                j.OutputDirectory = Directory.GetCurrentDirectory();
                m.OutputFileName = Path.GetFileNameWithoutExtension(o.InputFile) + ".wma";
            }
            else
            {
                string bla = Path.GetDirectoryName(o.OutputFile);

                if (!bla.Contains(@"\\"))
                    j.OutputDirectory = Directory.GetCurrentDirectory();
                else
                    j.OutputDirectory = Path.GetDirectoryName(o.OutputFile);

                m.OutputFileName = Path.GetFileName(o.OutputFile);
            }

            f.AudioProfile = a;
            m.OutputFormat = f;
            j.MediaItems.Add(m);
            j.CreateSubfolder = false;
            Console.WriteLine("encoding {0} to {1}", o.InputFile, o.OutputFile);
            j.EncodeProgress += new EventHandler<EncodeProgressEventArgs>(this.progr);
            j.EncodeCompleted += new EventHandler<EncodeCompletedEventArgs>(this.compl);

            j.Encode();
        }

        public static void Main(string[] args)
        {
            AppDomain.CurrentDomain.UnhandledException += CurrentDomainUnhandledException;

            var o = new Options();

            //o.Bitrate = 128;
            //o.Codec = WmaCodec.wma;
            //o.Mode = Mode.cbr;
            ////o.Quality = 10;
            //o.Rate = 44100;
            //o.Bits = 16;
            //o.Channels = 2;
            //o.InputFile = @"c:\temp\sgirl.wav";
            //o.OutputFile = @"bla.wma";
            //new Program(o);

            var parser = new CommandLine.Parser(with => with.HelpWriter = Console.Error);

            if (parser.ParseArgumentsStrict(args, o, () => Environment.Exit(-2)))
            {
                new Program(o);
            }

        }

        public void progr(object sender, EncodeProgressEventArgs e)
        {
            Console.Write("\r{0:0.00}%", e.Progress);
        }

        public void compl(object sender, EncodeCompletedEventArgs e)
        {
            Console.WriteLine("\nDone!");
        }

        static void CurrentDomainUnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            var err = (Exception)e.ExceptionObject;
            Console.Write(err.Message);
            Environment.Exit(1);
        }
    }
}
